package com.tuoheng.api.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.tuoheng.common.core.constant.SecurityHeadConstant;
import com.tuoheng.common.core.entity.UserEntity;
import com.tuoheng.common.core.utils.EncryptUtil;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.system.feign.feign.UserClient;
import com.tuoheng.system.feign.vo.UserInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @Author: 吴彬
 * @CreateTime: 2022-09-13  08:40
 * @Description: 获取gateway携带的身份信息前置过滤器
 * @Version: 1.0
 */
@Component
@Slf4j
public class AuthenticationFilter extends OncePerRequestFilter {

    @Autowired
    private UserClient userClient;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response,
                                    FilterChain filterChain) throws ServletException, IOException {
        String token = request.getHeader(SecurityHeadConstant.JSON_TOKEN);
        if (StringUtils.isNotBlank(token)){
            String json = EncryptUtil.decodeUTF8StringBase64(token);
            JSONObject jsonObject = JSON.parseObject(json);
            Object principal = jsonObject.getObject("principal",Object.class);
            //获取用户身份信息、权限信息
            UserEntity user = null;
            String[] authorities = null;
            /**
             * 判断携带的身份信息是用户实体还是client_id
             */
            if(principal instanceof UserEntity){//用户实体
                //获取用户身份信息、权限信息
                user = (UserEntity)principal;
                JSONArray tempJsonArray = jsonObject.getJSONArray("authorities");
                authorities =  tempJsonArray.toArray(new String[0]);
            }else{//client_id
                //根据client_id调用system服务获取用户实体信息
                JsonResult<UserInfo> result = userClient.getInfoByClientId(principal.toString());
                if (result.getCode() != JsonResult.SUCCESS ||  result.getData() == null || result.getData().getUserEntity() == null) {
                    log.error("未获取到【{}】对应的用户信息",principal.toString());
                    //解决response中writer乱码方案
                    response.setHeader("Content-Type", "application/json;charset=utf-8");
                    response.getWriter().print(JSON.toJSONString(result, SerializerFeature.WriteMapNullValue));
                    response.getWriter().flush();
                    return;
                }
                user = result.getData().getUserEntity();
                authorities =  result.getData().getPermissions().toArray(new String[0]);
            }
            //身份信息、权限信息填充到用户身份token对象中
            UsernamePasswordAuthenticationToken authenticationToken=new UsernamePasswordAuthenticationToken(user,null,
                    AuthorityUtils.createAuthorityList(authorities));
            //创建details
            authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
            //将authenticationToken填充到安全上下文
            SecurityContextHolder.getContext().setAuthentication(authenticationToken);
        }
        filterChain.doFilter(request,response);
    }

}
